#include <iostream>
#include <vector>
#include "svg.h"

void
svg_begin(double width, double height) {
    std::cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    std::cout << "<svg ";
    std::cout << "width='" << width << "' ";
    std::cout << "height='" << height << "' ";
    std::cout << "viewBox='0 0 " << width << " " << height << "' ";
    std::cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void
svg_end() {
    std::cout << "</svg>\n";
}

void
svg_text(double left, double baseline, std::string text) {
     std::cout << "<text x='" << left << "' y='"<< baseline << "' > " << text << " </text>";
}

void svg_rect(double x, double y, double width, double height){
        std::cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height << " ' stroke='#561A75' fill='#EDB1DB' ></rect>";
}

double
get_block_width() {
    double BLOCK_WIDTH;
    do {
        std::cerr << "Enter width of one historgamm block (from 3 to 30): ";
        std::cin >> BLOCK_WIDTH;
        if (BLOCK_WIDTH < 3) {
                std::cerr << "The block width cannot be less than 3px. Enter the value again.\n";
        }
        else if (BLOCK_WIDTH > 30) {
                std::cerr << "The block width cannot be more than 30px. Enter the value again.\n";
        }

    } while (BLOCK_WIDTH < 3 || BLOCK_WIDTH > 30);
    return BLOCK_WIDTH;
}

void
show_histogram_svg(const std::vector<std::size_t>& bins) {



    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    double BLOCK_WIDTH = get_block_width();

    const std::size_t MAX_ASTERISK = IMAGE_WIDTH - TEXT_WIDTH;
    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);
    double top = 0;

    std::size_t maxbin=bins[0];
    for (std::size_t i=0; i < bins.size(); i++){
        if (maxbin < bins[i]){
            maxbin=bins[i];
        }
    }

    if (maxbin<=76){
        for (std::size_t i=0; i < bins.size(); i++) {
            double bin_width = BLOCK_WIDTH * bins[i];
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bins[i]));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
            top += BIN_HEIGHT;
        }
        svg_end();
    } else {
        for (std::size_t i=0; i < bins.size(); i++) {
            double bin_width= MAX_ASTERISK * (static_cast<double>(bins[i]) / maxbin);
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bins[i]));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
            top += BIN_HEIGHT;
        }
        svg_end();
    }
}


